@@ -1,6 +1,6 @@
 # The MIT License
 #
-# Copyright (c) 2021 Sebastian Ramacher
+# Copyright (c) 2024 Sebastian Ramacher
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,13 +21,7 @@
 # THE SOFTWARE.
 
 try:
-    # introduced in Python 3.8
-    from typing import Literal
+    # introduced in Python 3.11
+    from typing import Never
 except ImportError:
-    from typing_extensions import Literal  # type: ignore
-
-try:
-    # introduced in Python 3.8
-    from typing import Protocol
-except ImportError:
-    from typing_extensions import Protocol  # type: ignore
+    from typing_extensions import Never  # type: ignore
